#!/usr/bin/env python3
"""
Module LISTVIEW -- GUI Tree/List View Widgets
Sub-Package GUI.WIDGETS of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

Defines the widget classes for tree view/list view widgets.
"""

import types

from plib.gui.defs import *
from plib.gui._widgets import _control, _helpers, _signal


class PListViewItemCols(_helpers._PTreeHelperItemCols):
    """Exposes columns of a list view item as a sequence.
    """
    
    def _get_listviewitem(self):
        return self._helperitem
    
    listviewitem = property(_get_listviewitem)
    
    def _get_data(self, index):
        return self.listviewitem._get_col(index)
    
    def _set_data(self, index, value):
        self.listviewitem._set_col(index, value)


class PListViewItemBase(_helpers._PTreeHelperItem):
    """List view item that looks like a sequence of 2-tuples.
    
    Each 2-tuple in the sequence is of the form
    (column-values, [list of child items]). The column-values
    is a list of strings; if the list view has only one column,
    the list has a single element.
    """
    
    colsclass = PListViewItemCols
    
    def __init__(self, parent, index, data=None):
        self._parent = parent
        self._depth = 0
        self._listview = self._listview_from_parent(parent)
        _helpers._PTreeHelperItem.__init__(self, parent, index, data)
    
    def _listview_from_parent(self, p):
        while not isinstance(p, self.listviewclass):
            p = p._parent
            self._depth += 1
        return p
    
    def _get_listview(self):
        return self._listview
    
    listview = property(_get_listview)
    
    def colcount(self):
        return self.listview.colcount()
    
    def _get_col(self, col):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def _set_col(self, col, value):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def expand(self):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError


class PListViewLabelsBase(_helpers._PHelperColLabels):
    
    def _get_listview(self):
        return self._helper
    
    listview = property(_get_listview)


class PListViewMixin(_helpers._PTreeHelper):
    """Mixin class for PListViewBase and PListBoxBase.
    
    Implements common behaviors.
    
    Includes convenience methods to handle common special cases.
    """
    
    def __init__(self, parent, labels=None, data=None):
        # Some GUI toolkits define a clear method but it
        # doesn't work reliably, so we insist on using
        # ours (note that we have to create the bound
        # method by hand)
        self.clear = types.MethodType(
            _helpers._PTreeHelper.clear, self)
        self._parent = parent
        if not hasattr(self.itemclass, 'listviewclass'):
            self.itemclass.listviewclass = self.__class__
        _helpers._PTreeHelper.__init__(self, labels, data)
    
    def colcount(self):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def extend_flat(self, items):
        """Adds list of multi-column items to list.
        
        Assumes no children, but multiple columns.
        """
        self.extend((item, []) for item in items)
    
    def extend_list(self, items):
        """Adds list of strings to list.
        
        Assumes a single column list box with no children.
        """
        self.extend(([item], []) for item in items)


class PListViewBase(_signal._PNotifyControl, PListViewMixin):
    """List view that looks like a list of child list view items.
    """
    
    signal = SIGNAL_LISTSELECTED
    
    def __init__(self, parent, labels=None, data=None, target=None):
        PListViewMixin.__init__(self, parent, labels, data)
        _signal._PNotifyControl.__init__(self, target)


class PListBoxBase(_control._PDialogControl, PListViewMixin):
    """List view specialized for use as a dialog control.
    """
    
    signal = SIGNAL_LISTSELECTED
    
    def __init__(self, parent, labels=None, data=None, target=None, geometry=None):
        PListViewMixin.__init__(self, parent, labels, data)
        _control._PDialogControl.__init__(self, target, geometry)



#!/usr/bin/env python3
"""
Module KDE4APP -- Python KDE Application Objects
Sub-Package GUI.TOOLKITS.KDE of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the KDE 4 GUI application objects.
"""

from PyQt4 import Qt as qt
from PyKDE4 import kdecore
from PyKDE4 import kdeui
import sip

from plib.gui.defs import *
from plib.gui._base import app

from ._kde4common import _PKDECommunicator, _kdemessagefuncs


def _int(button):
    if button is not None:
        return button
    else:
        return 0


class PKDEMessageBox(app.PMessageBoxBase):
    """Customized KDE message box.
    """
    
    questionmap = {
        answerYes: qt.QMessageBox.Yes,
        answerNo: qt.QMessageBox.No,
        answerCancel: qt.QMessageBox.Cancel,
        answerOK: qt.QMessageBox.Ok
    }
    
    def _messagebox(self, type, caption, text,
                    button1, button2=None, button3=None):
        
        return _kdemessagefuncs[type](self._parent, caption, text,
                                      _int(button1), _int(button2), _int(button3))


class PKDEFileDialog(app.PFileDialogBase):
    
    def openfiledialog(self, path, filter):
        return qt.QFileDialog.getOpenFileName(None, "Open", path, filter)
    
    def savefiledialog(self, path, filter):
        return qt.QFileDialog.getSaveFileName(None, "Save", path, filter)


def _kdeabout(name, version):
    return kdecore.KAboutData(name, "", kdecore.ki18n(name), version)


def _kdeparse(aboutdata):
    data = _kdeabout(aboutdata['name'], aboutdata['version'])
    data.setCopyrightStatement(kdecore.ki18n(aboutdata['copyright']))
    data.setLicenseText(kdecore.ki18n(aboutdata['license']))
    data.setShortDescription(kdecore.ki18n(aboutdata['description']))
    for dev in aboutdata['developers']:
        data.addAuthor(kdecore.ki18n(dev))
    data.setHomepage(aboutdata['website'])
    return data


def _kdelogo(icon, kaboutdata):
    qimg = qt.QImage(icon)
    qimg_scaled = qimg.scaled(qimg.width() * 2, qimg.height() * 2)
    # these incantations are to get around weirdness in KDE 4 setProgramLogo
    vptr = sip.voidptr(sip.unwrapinstance(qimg_scaled))
    vtype = int(qt.QVariant.Image)
    kaboutdata.setProgramLogo(qt.QVariant(vtype, vptr))


class _PKDEBaseMixin(_PKDECommunicator, kdeui.KMainWindow):
    """Mixin class for KDE base windows.
    """
    
    _closed = False  # default for guard to trap close from system menu, see below
    
    def _get_w(self):
        return self.width()
    w = property(_get_w)
    
    def _show_window(self):
        kdeui.KMainWindow.show(self)
    
    def _hide_window(self):
        kdeui.KMainWindow.hide(self)
    
    def set_caption(self, caption):
        self.setPlainCaption(caption)
    
    def _get_desktop_rect(self, primary=True):
        # Correctly handle virtual desktop across multiple screens
        desktop = self.app.desktop()
        l = desktop.x()
        t = desktop.y()
        w = desktop.width()
        h = desktop.height()
        if desktop.isVirtualDesktop() and primary:
            # Return the rect of the primary screen only
            i = desktop.primaryScreen()
            n = desktop.numScreens()
            w = w / n
            # NOTE: We have to check for i > 0 here because in some
            # cases (e.g., when running in a VirtualBox), Qt thinks
            # the desktop is "virtual" but there's only one screen and
            # desktop.primaryScreen returns 0 instead of 1.
            if i > 0:
                l += w * (i - 1)
        else:
            i = 0
            n = 1
        return i, n, l, t, w, h
    
    def sizetoscreen(self, maximized):
        if maximized:
            if self.shown:
                self.showMaximized()
            else:
                self._showMax = True
        else:
            i, n, l, t, w, h = self._get_desktop_rect()
            self.resize(
                w - self.sizeoffset,
                h - self.sizeoffset)
            self.move(l, t)
    
    def sizetoclient(self, clientwidth, clientheight):
        self.resize(clientwidth, clientheight)
    
    def center(self):
        i, n, l, t, w, h = self._get_desktop_rect()
        s = self.frameSize()  # FIXME: this appears to give wrong values!
        x, y = s.width(), s.height()
        self.move(l + (w - x) / 2, t + (h - y) / 2)
    
    def show_init(self):
        if hasattr(self, '_showMax'):
            self.showMaximized()
            del self._showMax
        else:
            kdeui.KMainWindow.show(self)
    
    def exit(self):
        # Guard traps a close other than from this method, so we don't throw
        # an exception if this method gets called after we close but before
        # shutdown (?? Why isn't this necessary in Qt?)
        if not self._closed:
            self.close()
    
    def closeEvent(self, event):
        # 'automagic' code for SIGNAL_QUERYCLOSE
        if self._canclose():
            self._emit_event(SIGNAL_CLOSING)
            self._closed = True  # set guard used above
            event.accept()
        else:
            event.ignore()
    
    def showEvent(self, event):
        self._emit_event(SIGNAL_SHOWN)
    
    def hideEvent(self, event):
        self._emit_event(SIGNAL_HIDDEN)


class PKDEBaseWindow(_PKDEBaseMixin, app.PBaseWindowBase):
    """Customized KDE base window class.
    """
    
    def __init__(self, parent, cls=None):
        _PKDEBaseMixin.__init__(self)
        app.PBaseWindowBase.__init__(self, parent, cls)
        self.setCentralWidget(self.clientwidget)
    
    def show_init(self):
        app.PBaseWindowBase.show_init(self)
        _PKDEBaseMixin.show_init(self)


class _PKDEMainMixin(_PKDEBaseMixin):
    """Mixin class for KDE top windows and main windows.
    """
    
    messageboxclass = PKDEMessageBox
    filedialogclass = PKDEFileDialog
    
    def set_iconfile(self, iconfile):
        self.setWindowIcon(kdeui.KIcon(qt.QIcon(qt.QPixmap(iconfile))))
    
    def _size_to_settings(self, width, height):
        self.resize(width, height)
    
    def _move_to_settings(self, left, top):
        self.move(left, top)
    
    def _get_current_geometry(self):
        p = self.pos()
        s = self.size()
        return p.x(), p.y(), s.width(), s.height()
    
    def choose_directory(self, curdir):
        return str(qt.QFileDialog.getExistingDirectory(
            self, "Select Folder", qt.QString(curdir)))
    
    def about(self):
        if self.aboutdata is not None:
            kaboutdata = _kdeparse(self.aboutdata)
            if 'icon' in self.aboutdata:
                _kdelogo(self.aboutdata['icon'], kaboutdata)
            kdeui.KAboutApplicationDialog(kaboutdata).exec_()
    
    def about_toolkit(self):
        kdeui.KHelpMenu(self).aboutKDE()


class PKDETopWindow(_PKDEMainMixin, app.PTopWindowBase):
    """Customized KDE top window class.
    """
    
    def __init__(self, parent, cls=None):
        _PKDEMainMixin.__init__(self)
        app.PTopWindowBase.__init__(self, parent, cls)
        self.setCentralWidget(self.clientwidget)
    
    def show_init(self):
        app.PTopWindowBase.show_init(self)
        _PKDEMainMixin.show_init(self)


class PKDEApplication(_PKDECommunicator, kdeui.KApplication,
                      app.PApplicationBase):
    """Customized KDE application class.
    """
    
    _local_loop = None
    
    def __init__(self, arglist=[], cls=None, use_mainwindow=False):
        if cls is None:
            klass = self.mainwidgetclass
        else:
            klass = cls
        
        if hasattr(klass, 'aboutdata') and (klass.aboutdata is not None):
            aboutdata = klass.aboutdata
        
        elif hasattr(klass, 'clientwidgetclass') and \
                hasattr(klass.clientwidgetclass, 'aboutdata'):
            
            aboutdata = klass.clientwidgetclass.aboutdata
        
        else:
            aboutdata = None
        
        if aboutdata:
            kaboutdata = _kdeparse(aboutdata)
        else:
            kaboutdata = _kdeabout("Unnamed", "0.0")
        
        # All the above because KDE requires this incantation first
        kdecore.KCmdLineArgs.init(kaboutdata)
        kdeui.KApplication.__init__(self)
        app.PApplicationBase.__init__(self, arglist, cls, use_mainwindow)
        self.mainwin = self.createMainWidget()
        #self.setMainWidget(self.mainwin)
        
        # 'automagic' signal connection
        self.setup_notify(SIGNAL_BEFOREQUIT, self.before_quit)
    
    def _eventloop(self):
        self.exec_()
    
    def process_events(self):
        self.processEvents()
    
    # For use when multiplexing with other event types (e.g.,
    # in a NotifierClient
    
    def enter_yield(self):
        if self._local_loop is None:
            self._local_loop = qt.QEventLoop()
            self._local_loop.exec_()
    
    def exit_yield(self):
        if self._local_loop is not None:
            self._local_loop.exit()
            del self._local_loop



#!/usr/bin/env python3
"""
PYIDSERVER-GUI.PY
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

GUI for Python implementation of IDServer. See
PYIDSERVER.PY for more information about the basic
"engine" involved. This GUI overlay to the engine is
intended to demonstrate some key features of the
``plib.gui`` sub-package, as well as the general Unix
programming model of "build a base engine first, which
can be driven by a rudimentary command-line interface;
then build a GUI on top of it." The program also shows
how to multiplex a GUI with socket events.

Key features:

  - The ``plib.gui.specs`` module is used to define
    the entire GUI in terms of Python lists and tuples.
    The only actual code involved is some setup after
    the GUI controls are instantiated, and the event
    handlers for the specific control events.
  
  - The event handlers themselves are "automagically"
    bound to their control event signals. This is done
    by making the name of the event handler bear a
    predictable relationship to the name of the control
    itself. The underlying code in the ``gui.PMainPanel``
    class then does the method lookups involved and
    binds the control signals to their targets. See the
    comments in the GUI spec definitions.
  
  - The GUI event loop and the socket I/O required for
    idserver are multiplexed; even if it takes some time
    for the remote server to respond with data, the GUI
    remains responsive, with no code required other than
    mixing in the ``NotifierClient`` class (we do add
    a few lines of code to the derived class to enable
    us to stop a query in progress from the GUI, but
    that is extra functionality; the multiplexing itself
    would work fine without it).
"""

import os

from plib.gui import __version__
from plib.gui import main as gui
from plib.gui import common
from plib.gui.defs import *

# No need to define custom actions since the "Ok" action
# doesn't get used anywhere else in this app (see the
# scrips-edit example program for a case where it does)
CAPTION_GO = "&Go"
ICON_GO = common.action_iconfile(ACTION_OK)
CAPTION_STOP = "&Stop"
ICON_STOP = common.action_iconfile(ACTION_CANCEL)
common.set_action_caption(ACTION_OK, CAPTION_GO)

from plib.gui import specs
specs.mainwidth = 400
specs.mainheight = 300
del specs
from plib.gui.specs import *

try:
    import pyidserver3 as pyidserver
except ImportError:
    from sysconfig import get_path
    
    from plib.stdlib.imp import import_from_path
    from plib.stdlib.postinstall import get_share_dir
    
    pyidserver = import_from_path(
        os.path.join(
            get_share_dir('plib3.stdlib', 'plib.stdlib'),
            'examples', 'pyidserver'
        ),
        'pyidserver3'
    )


# Monkeypatch idserver to multiplex with GUI event loop
# and add the ability to stop the query in progress

chat_replies = pyidserver.chat_replies


class chat_replies_gui(gui.NotifierClient, chat_replies):
    
    instance = None
    
    def __init__(self, addr, items, connect_fn=None, close_fn=None):
        chat_replies.__init__(self, addr, items,
                              connect_fn=connect_fn, close_fn=close_fn)
        chat_replies_gui.instance = self
        
        if hasattr(pyidserver, 'asyncore'):
            self.do_close = self.handle_close
        else:
            self.do_close = self.close
    
    def _clear_instance(self):
        chat_replies_gui.instance = None
    
    if hasattr(pyidserver, 'asyncore'):
        def handle_close(self):
            chat_replies.handle_close(self)
            self._clear_instance()
    
    else:
        def close(self):
            chat_replies.close(self)
            self._clear_instance()


pyidserver.chat_replies = chat_replies_gui


class IDServerFrame(gui.PMainPanel):
    
    aboutdata = {
        'name': "PyIDServer",
        'version': __version__,
        'copyright': "Copyright (C) 2008-2015 by Peter A. Donis",
        'license': "GNU General Public License (GPL) Version 2",
        'description': "A Python GUI for IDServer",
        'developers': ["Peter Donis"],
        'website': "http://www.peterdonis.net",
        'icon': os.path.join(os.path.split(os.path.realpath(__file__))[0],
                             "pyidserver.png")
    }
    
    maintitle = "PyIDServer"
    
    layout = LAYOUT_VERTICAL
    placement = (SIZE_CLIENTWRAP, MOVE_CENTER)
    
    childlist = [
        toplevel_panel(ALIGN_TOP, LAYOUT_HORIZONTAL, 'top', [
            interior_panel(ALIGN_JUST, LAYOUT_HORIZONTAL, 'left', [
                # No standard event handler for this edit box (the signal
                # we want to bind to, the Enter key, isn't the standard
                # one for an edit box, so we'll do the binding by hand
                # below); note that the automatic lookup code for event
                # handlers (see below) does nothing if no matching handler
                # is found (this makes declaring the GUI a lot easier since
                # you don't have to duplicate knowledge of which controls
                # have handlers, just declare the ones you want as methods)
                editbox('url')
            ]),
            interior_panel(ALIGN_RIGHT, LAYOUT_HORIZONTAL, 'right', [
                # This action button has a non-standard name, so we have to
                # specify it; the ``go`` method of the main panel then becomes
                # the target (buttons and action buttons are the only spec
                # controls that automatically assume they have targets; this
                # seems reasonable since that's what buttons are for)
                action_button(ACTION_OK, 'go')
            ])
        ]),
        toplevel_panel(ALIGN_JUST, LAYOUT_VERTICAL, 'main', [
            midlevel_panel(ALIGN_TOP, LAYOUT_HORIZONTAL, 'header', [
                toplevel_panel(ALIGN_LEFT, LAYOUT_HORIZONTAL, 'controls', [
                    # Each control below will automatically look for its
                    # event handler; the target method for check boxes is
                    # ``<name>_toggled``, where ``<name>`` is the base name of
                    # the control (the 2nd parameter)--note that in the specs
                    # code the actual full name of the control will be
                    # ``checkbox_<name>``, to avoid name collisions between
                    # controls of different types; for the combo box, the
                    # handler method name will be ``<name>_selected``
                    checkbox("DNS Only", 'dnsonly'),
                    checkbox("Set Protocol", 'protocol'),
                    sorted_combobox('protocol', pyidserver.protocols.keys()),
                    checkbox("Set Port", 'portnum'),
                    # No handler for the numedit, just need to keep it at a
                    # fixed size
                    numeditbox('portnum', expand=False)
                ]),
                padding('main')
            ]),
            interior_panel(ALIGN_JUST, LAYOUT_VERTICAL, 'body', [
                # No event handler for the text control (since it's
                # display-only anyway)
                textcontrol('output')
            ])
        ]),
        toplevel_panel(ALIGN_BOTTOM, LAYOUT_HORIZONTAL, 'bottom', [
            padding('bottom'),
            # These action buttons all have their standard names, so no
            # additional parameters are needed; each one's target will be the
            # method with its name: ``about``, ``about_toolkit``, and
            # ``exit`` (note that the ``exit`` method is in the parent, which
            # is the main window, not the main panel itself; the automatic
            # lookup method finds it by traversing the parent tree if the
            # method is not found in the panel)
            action_button(ACTION_ABOUT),
            action_button(ACTION_ABOUTTOOLKIT),
            action_button(ACTION_EXIT)
        ])
    ]
    
    query_in_progress = False
    
    def _createpanels(self):
        # Create child widgets
        super(IDServerFrame, self)._createpanels()
        
        # The function we're wrapping with our GUI
        self.func = pyidserver.run_main
        
        # Controls affected by the DNS Only checkbox
        self.dnsonly_controls = (
            (self.checkbox_protocol, self.combo_protocol),
            (self.checkbox_portnum, self.edit_portnum)
        )
        
        # Controls independent of DNS Only checkbox
        self.other_controls = (
            self.edit_url, self.checkbox_dnsonly)
        
        # Adjust some widget parameters that couldn't be set in constructors
        self.edit_url.setup_notify(SIGNAL_ENTER, self.go)
        
        self.checkbox_dnsonly.checked = self.default_dnsonly()
        self.dnsonly_toggled()  # technically not needed, put in for completeness
        
        self.combo_protocol.set_current_text(
            self.default_protocol() or pyidserver.PROTO_DEFAULT)
        self.protocol_toggled()
        
        self.edit_portnum.edit_text = str(self.default_portnum())
        self.portnum_toggled()
        
        self.text_output.set_font("Courier New")
        
        # Set up output file-like object here for convenience
        self.outputfile = gui.PTextFile(self.text_output)
        
        # Start with keyboard focus in the URL text entry
        self.edit_url.set_focus()
    
    def default_dnsonly(self):
        return self.func.__defaults__[2]
    
    def default_protocol(self):
        return self.func.__defaults__[3]
    
    def default_portnum(self):
        return self.func.__defaults__[4]
    
    def dnsonly_toggled(self):
        """Called when the dns_only checkbox is checked/unchecked.
        
        Only enable protocol and port controls if not DNS only.
        """
        
        enable = not self.checkbox_dnsonly.checked
        for ctrl, subctrl in self.dnsonly_controls:
            ctrl.enabled = enable
            subctrl.enabled = enable and ctrl.checked
    
    def protocol_toggled(self):
        """Called when the protocol checkbox is checked/unchecked.
        
        Sync protocol combo enable with check box."""
        self.combo_protocol.enabled = self.checkbox_protocol.checked
    
    def protocol_selected(self, index):
        """Called when a protocol combo selection is made.
        
        For now, just prints a diagnostic showing the signal response.
        """
        print(index, self.combo_protocol[index])
    
    def portnum_toggled(self):
        """Called when the portnum checkbox is checked/unchecked.
        
        Sync portnum edit enable with check box.
        """
        self.edit_portnum.enabled = self.checkbox_portnum.checked
    
    def go(self):
        """Called when the Go/Stop button is pushed.
        
        Execute the idserver query, or stop a query in progress.
        For executing a query, this method is also called when the
        Enter key is pressed while in the URL edit box.
        """
        
        if self.query_in_progress:
            if chat_replies_gui.instance:
                # Shut down the async I/O
                chat_replies_gui.instance.do_close()
            else:
                # Async I/O hasn't started yet, can't abort
                return  # FIXME: how to abort from syscalls here?
        
        else:
            # Clear output
            self.outputfile.truncate(0)
            
            # Check URL
            url = self.edit_url.edit_text
            if len(url) < 1:
                self.outputfile.write("Error: No URL entered.")
                self.outputfile.flush()
                return
            
            # Fill in arguments that user selected, if any
            dns_only = self.checkbox_dnsonly.checked
            if self.checkbox_protocol.checked:
                protocol = self.combo_protocol.current_text()
            else:
                protocol = self.default_protocol()
            if self.checkbox_portnum.checked:
                portnum = int(self.edit_portnum.edit_text)
            else:
                portnum = self.default_portnum()
            
            # Now execute
            for ctrl, subctrl in self.dnsonly_controls:
                ctrl.enabled = subctrl.enabled = False
            for ctrl in self.other_controls:
                ctrl.enabled = False
            self.button_go.set_caption(CAPTION_STOP)
            self.button_go.set_icon(ICON_STOP)
            self.query_in_progress = True
            
            self.func(url, outfile=self.outputfile,
                      dns_only=dns_only, protocol=protocol, portnum=portnum)
        
        # Either we're done, or we've stopped a query in progress
        self.query_in_progress = False
        self.button_go.set_caption(CAPTION_GO)
        self.button_go.set_icon(ICON_GO)
        for ctrl in self.other_controls:
            ctrl.enabled = True
        self.dnsonly_toggled()
    
    def acceptclose(self):
        # If query is in progress, shut it down before closing
        if chat_replies_gui.instance:
            chat_replies_gui.instance.do_close()
        return True


if __name__ == "__main__":
    # Our client frame will be wrapped in a ``PTopWindow``
    gui.runapp(IDServerFrame)



#!/usr/bin/env python3
"""
Module KDE4COMBO -- Python KDE Combo Box Widgets
Sub-Package GUI.TOOLKITS.KDE of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the KDE 4 GUI objects for combo boxes.
"""

from PyQt4 import Qt as qt
from PyKDE4 import kdeui

from plib.gui._widgets import combo

from ._kde4common import _PKDEMeta, _PKDEWidget


class PKDEComboBox(_PKDEWidget, kdeui.KComboBox, combo.PComboBoxBase, metaclass=_PKDEMeta):
    
    widget_class = kdeui.KComboBox
    
    def __init__(self, parent, sequence=None, target=None, geometry=None):
        kdeui.KComboBox.__init__(self, False, parent)
        self.setSizePolicy(qt.QSizePolicy.Fixed, qt.QSizePolicy.Fixed)
        combo.PComboBoxBase.__init__(self, sequence, target, geometry)
    
    def set_width(self, width):
        self.resize(width, self.height())
        # Qt buttons don't appear to fully respect QSizePolicy.Fixed
        self.setMinimumWidth(width)
    
    def current_text(self):
        return str(self.currentText())
    
    # Note that there's no quick override for set_current_text in KDE; the
    # corresponding method to the above doesn't do what we want (it changes
    # the stored text in the combo instead of selecting the text we give it)
    
    def current_index(self):
        return self.currentIndex()
    
    def set_current_index(self, index):
        self.setCurrentIndex(index)
    
    def count(self, value):
        # Method name collision, we want it to be the Python sequence
        # count method here.
        return combo.PComboBoxBase.count(self, value)
    
    def _indexlen(self):
        # Let this method access the KDE combo box count method.
        return kdeui.KComboBox.count(self)
    
    def _get_data(self, index):
        return str(self.itemText(index))
    
    def _set_data(self, index, value):
        self.setItemText(index, str(value))
    
    def _add_data(self, index, value):
        self.insertItem(index, str(value))
    
    def _del_data(self, index):
        self.removeItem(index)



#!/usr/bin/env python3
"""
Module KDE4BUTTON -- Python KDE Button Widgets
Sub-Package GUI.TOOLKITS.KDE of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the KDE 4 GUI objects for button widgets.
"""

from PyQt4 import Qt as qt
from PyKDE4 import kdeui

from plib.gui._widgets import button

from ._kde4common import _PKDEWidget, _kdestandardicons


class _PKDEButtonMixin(object):
    
    def set_caption(self, caption):
        self.setText(caption)


class PKDEButton(_PKDEButtonMixin, _PKDEWidget, kdeui.KPushButton,
                 button.PButtonBase):
    
    widget_class = kdeui.KPushButton
    
    def __init__(self, parent, caption=None, pxname=None,
                 target=None, geometry=None):
        
        kdeui.KPushButton.__init__(self, parent)
        self.setSizePolicy(qt.QSizePolicy.Fixed, qt.QSizePolicy.Fixed)
        # See if we're an action button with a standard icon
        if isinstance(caption, int) and (pxname == ""):
            key = caption
        elif isinstance(pxname, int):
            key = pxname
        else:
            key = None
        if key in _kdestandardicons:
            self.setIcon(kdeui.KIcon(_kdestandardicons[key]))
            pxname = None
        # Now do standard button initialization
        button.PButtonBase.__init__(self, caption, pxname, target, geometry)
    
    def set_icon(self, pxname):
        self.setIcon(kdeui.KIcon(qt.QIcon(qt.QPixmap(pxname))))
    
    def set_width(self, width):
        self.resize(width, self.height())
        # Qt buttons don't appear to fully respect QSizePolicy.Fixed
        self.setMinimumWidth(width)


class PKDECheckBox(_PKDEButtonMixin, _PKDEWidget, qt.QCheckBox,
                   button.PCheckBoxBase):
    
    fn_get_checked = 'isChecked'
    fn_set_checked = 'setChecked'
    
    widget_class = qt.QCheckBox
    
    def __init__(self, parent, caption=None, pxname=None, tristate=False,
                 target=None, geometry=None):
        
        qt.QCheckBox.__init__(self, parent)
        self.setSizePolicy(qt.QSizePolicy.Fixed, qt.QSizePolicy.Fixed)
        button.PCheckBoxBase.__init__(self, caption, pxname, tristate,
                                      target, geometry)
    
    def set_icon(self, pxname):
        # FIXME: pixmaps on checkboxes?
        pass
    
    def make_tristate(self):
        self.setTriState(True)



#!/usr/bin/env python3
"""
Module DIALOGS -- Dialog Classes for GUI
Sub-Package GUI of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

Defines dialogs for use with PLIB3.GUI applications. The
PDialogBase class implements the core functionality to
make it easy to integrate a dialog with a PLIB3.GUI
application. The following specific dialogs are also
included:

- PPrefsDialog: automatically constructs a dialog to
  allow editing of the preferences stored in a PIniFile.
"""

from plib.stdlib.ini.defs import *

from plib.gui import main as gui
from plib.gui import specs
from plib.gui.defs import *

# NOTE: I tried using "actual" dialog classes for this
# (e.g., qt.QDialog), but found weird behavior with the
# automated GUI generation code; so for now it's easier
# just to use PTopWindow and specialize it for dialogs.
# The only function this takes away (until I see a need
# to figure out how to do it) is modal dialogs--any dialog
# using this code will be modeless. For the use cases I'm
# interested in, that's not an issue; YMMV.


class PDialogBase(gui.PBaseWindow):
    """Base class for dialogs.
    
    Base class for dialogs; assumes that self.defs
    exists and contains a list of control definitions
    for use by a PAutoPanel. Provides the showdialog
    method to construct the controls (if not already
    constructed) and show the dialog; provides the
    hidedialog method to be linked to buttons in the
    dialog that should hide it.
    """
    
    defaultcaption = "Dialog"
    placement = (SIZE_CLIENTWRAP, MOVE_CENTER)
    
    def __init__(self, parent, panelclass):
        self._dirty = False
        self._modified = False
        gui.PBaseWindow.__init__(self, parent, panelclass)
        
        # Catch the main window closing signal so we can close ourselves
        # (some GUI toolkits don't actually require this, but we do it in
        # all cases for safety and tidiness)
        parent.setup_notify(SIGNAL_CLOSING, self.closedialog)
    
    def _populate_data(self):
        """Populate the dialog with data.
        """
        pass
    
    def _retrieve_data(self):
        """Retrieve data from the dialog.
        """
        pass
    
    def data_changed(self):
        """Widget change signals should call this method.
        """
        self._dirty = True
    
    def showdialog(self):
        """Populate dialog with data, then show it.
        """
        self._populate_data()
        if self.shown:
            self._show_window()
        else:
            self.show_init()
    
    def hidedialog(self):
        """Hide dialog (but don't destroy).
        """
        self._hide_window()
    
    def closedialog(self):
        """Close dialog.
        """
        self.exit()
    
    def button_apply(self):
        """Retrieve dialog data, but keep open.
        """
        if self._dirty:
            self._retrieve_data()
            self._dirty = False
            self._modified = True
    
    def button_ok(self):
        """Retrieve dialog data, then hide it.
        """
        self.button_apply()
        self.hidedialog()
    
    def button_cancel(self):
        """Revert dialog data to original, then hide it.
        """
        if self._dirty:
            self._populate_data()
            self._dirty = False
        self.hidedialog()


# FIXME: Make this code re-entrant, so multiple instances can use this
# same module (the obvious thing is to make the items below into class
# attributes, but doing that seems to break the automatic reading of
# INI file section/option names into variables)

PrefsSections = []  # this will be filled in dynamically by PPrefsDialog

PrefsButtonPanel = [
    specs.get_padding('buttons'),
    specs.get_action_button(ACTION_APPLY, 'button_apply'),
    specs.get_action_button(ACTION_OK),
    specs.get_action_button(ACTION_CANCEL)
]


class PPrefsPanel(gui.PAutoPanel):
    """Preferences dialog top-level panel.
    
    Panel to serve as the overall "frame" of the preferences dialog.
    The _childlist class field should be modified by PPrefsDialog before
    instantiating this class.
    """
    
    childlist = [
        specs.get_toplevel_panel(PrefsButtonPanel,
                                 ALIGN_BOTTOM, LAYOUT_HORIZONTAL,
                                 'buttons')
    ]
    
    def __init__(self, parent):
        gui.PAutoPanel.__init__(self, parent,
                                layout=LAYOUT_VERTICAL,
                                margin=specs.framemargin,
                                spacing=specs.panelspacing)


# Subclass dialog controls to add generic get_value and set_value methods

class PPrefsCheckBox(gui.PCheckBox):
    
    def get_value(self):
        return self.checked
    
    def set_value(self, value):
        self.checked = value


class PPrefsComboBox(gui.PComboBox):
    
    def get_value(self):
        return self.current_text()
    
    def set_value(self, value):
        self.set_current_text(value)


class PPrefsEditBox(gui.PEditBox):
    
    def get_value(self):
        return self.edit_text
    
    def set_value(self, value):
        self.edit_text = value


class PPrefsNumEditBox(gui.PEditBox):
    
    def get_value(self):
        return int(self.edit_text)
    
    def set_value(self, value):
        self.edit_text = str(value)


# Map data types to appropriate controls

controltypes = {
    INI_INTEGER: [
        PPrefsNumEditBox,
        (),
        {'expand': False, 'geometry': specs.numeditgeometry()}
    ],
    INI_BOOLEAN: [
        PPrefsCheckBox,
        (),
        {}
    ],
    INI_STRING: [
        PPrefsEditBox,
        (),
        {'geometry': specs.editgeometry()}
    ]
}


# Utility function to return INI option panel spec

def get_ini_panel(otype, label, name, target):
    klass, args, kwargs = controltypes[otype]
    kwargs['target'] = target
    if otype == INI_BOOLEAN:
        kwargs['caption'] = label
    result = [(klass, args, kwargs, name)]
    if otype != INI_BOOLEAN:
        result.insert(0, specs.get_label(label, name))
    return specs.get_interior_panel(result,
                                    ALIGN_TOP, LAYOUT_HORIZONTAL,
                                    name)


class PPrefsDialog(PDialogBase):
    """Auto-constructing preferences dialog.
    
    Preferences dialog: takes a parent PMainWindow and
    a PIniFile; constructs a dialog for editing the
    options in the PIniFile, and adds an action to the
    PMainWindow to display the dialog.
    """
    
    defaultcaption = "Preferences"
    panelclass = PPrefsPanel
    
    def __init__(self, parent, inifile, labels, style=SECTION_TAB):
        self._inifile = inifile
        self._labels = labels
        self._style = style
        self.data_map = {}
        
        if hasattr(parent, 'defaultcaption'):
            self.defaultcaption = "{} {}".format(parent.defaultcaption,
                                                 self.defaultcaption)
        
        # Build the specs from the INI file
        self.spec_sections(PrefsSections)
        
        # Now get the appropriate widget for the main panel and patch it in
        if style == SECTION_GROUPBOX:
            PrefsMainPanel = PrefsSections
        else:
            PrefsMainPanel = [
                specs.get_tabwidget(PrefsSections, 'main')
            ]
        self.panelclass.childlist.insert(0,
                                         specs.get_toplevel_panel(
                                         PrefsMainPanel,
                                         ALIGN_JUST, LAYOUT_VERTICAL,
                                         'main'))
        
        # This will now construct the dialog based on the specs
        PDialogBase.__init__(self, parent, self.panelclass)
        
        # TODO: Fix sizing in wx
    
    def showdialog(self):
        """Populate dialog with data, then show it.
        
        Note that if our parent is a PTopWindow that is storing
        its coordinates in settings, we have to update the
        settings variables first so the shown dialog will reflect
        the current state.
        """
        if self._parent:
            self._parent.get_coordinates()
        PDialogBase.showdialog(self)
    
    def button_apply(self):
        """Retrieve dialog data, but keep open.
        
        Note that if our parent is a PTopWindow that is storing
        its coordinates in settings, and.
        """
        PDialogBase.button_apply(self)
        if self._parent:
            self._parent.set_coordinates()
    
    def closedialog(self):
        """Save preferences if modified.
        """
        if self._modified:
            self._inifile.writeini()
        PDialogBase.closedialog(self)
    
    def _iterate_data(self, scallback, ocallback):
        if scallback is None:
            # We're populating or retrieving, need the actual panel widget
            main = self.clientwidget
        else:
            # We're specifying, need the spec list
            main = self._groups
        inifile = self._inifile
        labels = self._labels
        
        for sname, opts in inifile._optionlist:
            if scallback is not None:
                scallback(main, inifile, labels, sname)
            for opt in opts:
                attrname = getter = setter = None
                if len(opt) > 4:
                    oname, otype, odefault, getter, setter = opt
                elif len(opt) > 3:
                    oname, otype, odefault, attrname = opt
                else:
                    oname, otype, odefault = opt
                if attrname is None:
                    attrname = "{}_{}".format(sname, oname)
                if ocallback is not None:
                    ocallback(main, inifile, labels, attrname,
                              otype, getter, setter)
            if scallback is not None:
                scallback(main, inifile, labels, sname, True)
        
        if (scallback is not None) and (self._style == SECTION_GROUPBOX):
            self._groups.append(specs.get_padding('end'))
    
    def _current(self):
        if self._style == SECTION_GROUPBOX:
            return self._currentgroup[1][1]
        else:
            return self._currentgroup[1][0]
    
    def _spec_section(self, groups, inifile, labels, sname, done=False):
        # Add panel or group for INI file section
        if done:
            self._current().append(specs.get_padding(sname))
            if self._currentgroup is not None:
                groups.append(self._currentgroup)
        else:
            if self._style == SECTION_GROUPBOX:
                self._currentgroup = specs.get_groupbox([],
                                                        labels[sname],
                                                        sname)
            else:
                self._currentgroup = (
                    labels[sname],
                    specs.get_tab_panel([],
                                        ALIGN_TOP, LAYOUT_VERTICAL,
                                        sname)
                )
    
    def _spec_option(self, groups, inifile, labels, attrname,
                     otype, getter, setter):
        # Add individual INI file option to current panel/section
        self._current().append(get_ini_panel(otype,
                               labels[attrname], attrname, self.data_changed))
    
    def spec_sections(self, groups):
        """Generate the specs for the main panel from the INI file.
        """
        self._groups = groups
        self._currentgroup = None
        self._iterate_data(self._spec_section, self._spec_option)
        del self._currentgroup
        del self._groups
    
    def _populate_option(self, mainpanel, inifile, labels, attrname,
                         otype, getter, setter):
        # Populate control for INI file option
        control = getattr(mainpanel, attrname)
        if getter is not None:
            value = getter()
        else:
            value = getattr(inifile, attrname)
        control.set_value(value)
    
    def _populate_data(self):
        """Populate dialog data from PIniFile object.
        """
        self._iterate_data(None, self._populate_option)
    
    def _retrieve_option(self, mainpanel, inifile, labels, attrname,
                         otype, getter, setter):
        # Get GUI value for INI file option
        control = getattr(mainpanel, attrname)
        value = control.get_value()
        if setter is not None:
            setter(value)
        else:
            setattr(inifile, attrname, value)
    
    def _retrieve_data(self):
        """Set PIniFile options from dialog data.
        """
        self._iterate_data(None, self._retrieve_option)



#!/usr/bin/env python3
"""
Module PYSIDEAPP -- Python PySide Application Objects
Sub-Package GUI.TOOLKITS.PYSIDE of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the PySide GUI application objects.
"""

from PySide import QtGui as qt, QtCore as qtc

from plib.gui.defs import *
from plib.gui._base import app

from ._pysidecommon import _PQtCommunicator, _PQtWidgetMeta, _qtmessagefuncs


def _int(button):
    if button is not None:
        return button
    else:
        return 0


class PQtMessageBox(app.PMessageBoxBase):
    """Customized Qt message box.
    """
    
    questionmap = {
        answerYes: qt.QMessageBox.Yes,
        answerNo: qt.QMessageBox.No,
        answerCancel: qt.QMessageBox.Cancel,
        answerOK: qt.QMessageBox.Ok
    }
    
    def _messagebox(self, type, caption, text,
                    button1, button2=None, button3=None):
        
        buttons = default = button1
        if button2 is not None:
            buttons |= button2
            if button3 is not None:
                buttons |= button3
                default = button3
            else:
                default = button2
        
        return _qtmessagefuncs[type](self._parent, caption, text,
                                     buttons, default)


class PQtFileDialog(app.PFileDialogBase):
    
    def openfiledialog(self, path, filter):
        return qt.QFileDialog.getOpenFileName(None, "Open",
                                              path, filter)[0]
    
    def savefiledialog(self, path, filter):
        return qt.QFileDialog.getSaveFileName(None, "Save",
                                              path, filter)[0]


class PQtAboutDialog(app.PAboutDialogBase):
    
    attrnames = [
        'name',
        'version',
        'copyright',
        'license',
        'description',
        'developers',
        'website',
        'icon'
    ]
    
    formatstr = "{aname} {aversion}\n{adescription}\n{acopyright}\n{adevelopers}\n{awebsite}"
    
    def __getattribute__(self, name):
        # Here we have to modify the hack somewhat
        attrnames = object.__getattribute__(self, 'attrnames')
        if name in attrnames:
            object.__getattribute__(self, '__dict__')['temp'] = name
            name = 'store'
        return object.__getattribute__(self, name)
    
    def store(self, data):
        name = self.temp
        del self.temp
        if name == 'developers':
            data = ", ".join(data)
        setattr(self, "a{}".format(name), data)
    
    def display(self):
        caption = "About {}".format(self.aname)
        # Leave out icon here (setting it in set_iconfile below is enough, and
        # including it here will raise an exception if there is no icon)
        body = self.formatstr.format(**dict(
            ("a{}".format(name), getattr(self, "a{}".format(name)))
            for name in self.attrnames if name != 'icon'))
        qt.QMessageBox.about(self.mainwidget, caption, body)


class _PQtBaseMixin(_PQtCommunicator, qt.QMainWindow):
    """Mixin class for Qt base windows.
    """
    
    event_signals = (SIGNAL_CLOSING, SIGNAL_SHOWN, SIGNAL_HIDDEN)
    
    def _get_w(self):
        return self.width()
    w = property(_get_w)
    
    def _show_window(self):
        qt.QMainWindow.show(self)
    
    def _hide_window(self):
        qt.QMainWindow.hide(self)
    
    def set_caption(self, caption):
        self.setWindowTitle(caption)
    
    def _get_desktop_rect(self, primary=True):
        # Correctly handle virtual desktop across multiple screens
        desktop = self.app.desktop()
        l = desktop.x()
        t = desktop.y()
        w = desktop.width()
        h = desktop.height()
        if desktop.isVirtualDesktop() and primary:
            # Default to centering on the primary screen
            i = desktop.primaryScreen()
            n = desktop.numScreens()
            w = w / n
            # NOTE: We have to check for i > 0 here because in some
            # cases (e.g., when running in a VirtualBox), Qt thinks
            # the desktop is "virtual" but there's only one screen and
            # desktop.primaryScreen returns 0 instead of 1.
            if i > 0:
                l += w * (i - 1)
        else:
            i = 0
            n = 1
        return i, n, l, t, w, h
    
    def sizetoscreen(self, maximized):
        if maximized:
            if self.shown:
                self.showMaximized()
            else:
                self._showMax = True
        else:
            i, n, l, t, w, h = self._get_desktop_rect()
            self.resize(
                w - self.sizeoffset,
                h - self.sizeoffset)
            self.move(l, t)
    
    def sizetoclient(self, clientwidth, clientheight):
        self.resize(clientwidth, clientheight)
    
    def center(self):
        i, n, l, t, w, h = self._get_desktop_rect()
        s = self.frameSize()  # FIXME: this appears to give wrong values!
        x, y = s.width(), s.height()
        self.move(l + (w - x) / 2, t + (h - y) / 2)
    
    def show_init(self):
        if hasattr(self, '_showMax'):
            self.showMaximized()
            del self._showMax
        else:
            qt.QMainWindow.show(self)
    
    def exit(self):
        self.close()
    
    def closeEvent(self, event):
        # 'automagic' code for SIGNAL_QUERYCLOSE
        if self._canclose():
            self._emit_event(SIGNAL_CLOSING)
            event.accept()
        else:
            event.ignore()
    
    def showEvent(self, event):
        self._emit_event(SIGNAL_SHOWN)
    
    def hideEvent(self, event):
        self._emit_event(SIGNAL_HIDDEN)


class PQtBaseWindow(_PQtBaseMixin, app.PBaseWindowBase, metaclass=_PQtWidgetMeta):
    """Customized Qt base window class.
    """
    
    def __init__(self, parent, cls=None):
        _PQtBaseMixin.__init__(self)
        app.PBaseWindowBase.__init__(self, parent, cls)
        self.setCentralWidget(self.clientwidget)
    
    def show_init(self):
        app.PBaseWindowBase.show_init(self)
        _PQtBaseMixin.show_init(self)


class _PQtMainMixin(_PQtBaseMixin):
    """Mixin class for Qt top windows and main windows.
    """
    
    messageboxclass = PQtMessageBox
    filedialogclass = PQtFileDialog
    aboutdialogclass = PQtAboutDialog
    
    def set_iconfile(self, iconfile):
        self.setWindowIcon(qt.QIcon(qt.QPixmap(iconfile)))
    
    def _size_to_settings(self, width, height):
        self.resize(width, height)
    
    def _move_to_settings(self, left, top):
        self.move(left, top)
    
    def _get_current_geometry(self):
        p = self.pos()
        s = self.size()
        return p.x(), p.y(), s.width(), s.height()
    
    def choose_directory(self, curdir):
        return str(qt.QFileDialog.getExistingDirectory(
            self, "Select Folder", curdir))


class PQtTopWindow(_PQtMainMixin, app.PTopWindowBase, metaclass=_PQtWidgetMeta):
    """Customized Qt top window class.
    """
    
    def __init__(self, parent, cls=None):
        _PQtMainMixin.__init__(self)
        app.PTopWindowBase.__init__(self, parent, cls)
        self.abouttoolkitfunc = self.app.aboutQt
        self.setCentralWidget(self.clientwidget)
    
    def show_init(self):
        app.PTopWindowBase.show_init(self)
        _PQtMainMixin.show_init(self)


class PQtApplication(_PQtCommunicator, qt.QApplication, app.PApplicationBase):
    """Customized Qt application class.
    """
    
    _local_loop = None
    
    def __init__(self, arglist=[], cls=None, use_mainwindow=False):
        qt.QApplication.__init__(self, arglist)
        app.PApplicationBase.__init__(self, arglist, cls, use_mainwindow)
        self.mainwin = self.createMainWidget()
        #self.setMainWidget(self.mainwin)
        
        # 'automagic' signal connection
        self.setup_notify(SIGNAL_BEFOREQUIT, self.before_quit)
    
    def _eventloop(self):
        self.exec_()
    
    def process_events(self):
        self.processEvents()
    
    # For use when multiplexing with other event types (e.g.,
    # in a NotifierClient
    
    def enter_yield(self):
        if self._local_loop is None:
            self._local_loop = qtc.QEventLoop()
            self._local_loop.exec_()
    
    def exit_yield(self):
        if self._local_loop is not None:
            self._local_loop.exit()
            del self._local_loop



#!/usr/bin/env python3
"""
Module DEFS -- Common GUI Definitions
Sub-Package GUI of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

Defines common constants and functions used by various GUI modules.
"""

import sys

# general constants
default_font_size = 10

# constants for referring to GUI toolkits
#GUI_QT = 1
#GUI_GTK = 2
GUI_WX = 3
#GUI_KDE = 4
GUI_QT4 = 5
GUI_KDE4 = 6
GUI_PYSIDE = 7

# message box types
MBOX_INFO = 0
MBOX_WARN = 1
MBOX_ERROR = 2
MBOX_QUERY = 3

# constants for message box responses
answerNone = 0
answerYes = 1
answerNo = 2
answerCancel = 3
answerOK = 4

# constants for alignment
ALIGN_LEFT = 1
ALIGN_CENTER = 2
ALIGN_RIGHT = 3
ALIGN_TOP = 4
ALIGN_BOTTOM = 5
ALIGN_JUST = 9

# Layout constants
LAYOUT_NONE = 0
LAYOUT_HORIZONTAL = 1
LAYOUT_VERTICAL = 2

# Panel style constants
PANEL_NONE = 0
PANEL_BOX = 1
PANEL_RAISED = 2
PANEL_SUNKEN = 3

# Prefs dialog section style constants
SECTION_TAB = 0  # this is the default
SECTION_GROUPBOX = 1

# constants for signals
SIGNAL_ACTIVATED = 10  # widget has received focus
SIGNAL_CLICKED = 101  # widget has been clicked
SIGNAL_TOGGLED = 102  # on/off widget has been toggled
SIGNAL_SELECTED = 151  # item in widget has been selected; handler must take index param
SIGNAL_LISTSELECTED = 161  # list view item has changed; handler must take item param
SIGNAL_CELLSELECTED = 171  # table current cell has changed; handler must take row, col params
SIGNAL_FOCUS_IN = 191  # widget has received keyboard focus
SIGNAL_FOCUS_OUT = 195  # widget has lost keyboard focus
SIGNAL_CELLCHANGED = 301  # table cell text has changed; handler must take row, col params for changed cell
SIGNAL_TEXTCHANGED = 401  # text in edit control has been changed
SIGNAL_TEXTSTATECHANGED = 411  # state of edit control has changed
SIGNAL_EDITCHANGED = 450  # text in edit box has been changed
SIGNAL_ENTER = 490  # enter/return key has been pressed while widget has focus
SIGNAL_TABCHANGED = 501  # new tab has been selected; handler must take int param for new index
SIGNAL_CLOSING = 931  # widget close has been accepted
SIGNAL_SHOWN = 941  # widget has been shown
SIGNAL_HIDDEN = 951  # widget has been hidden

# these signal constants are for internal use only
SIGNAL_CHECKTOGGLED = 109  # internal check box toggled signal, filtered to SIGNAL_TOGGLED
SIGNAL_KEYPRESSED = 499  # internal key pressed signal, filtered to SIGNAL_ENTER
SIGNAL_TABCURRENTCHANGED = 599  # internal tab changed signal that gets re-interpreted
SIGNAL_NOTIFIER = 801  # socket notifier has received an event notification
SIGNAL_TABLECHANGED = 831  # internal table changed event that gets re-interpreted
SIGNAL_WIDGETCHANGED = 901  # widget has been changed (not including above specific changes)
SIGNAL_SHOWEVENT = 945  # internal window show/hide event that gets re-interpreted
SIGNAL_QUERYCLOSE = 991  # widget is asking permission to close
SIGNAL_BEFOREQUIT = 999  # app is about to quit

# constants for action flags, used as keys
ACTION_FILENEW = 5
ACTION_FILEOPEN = 10
ACTION_FILESAVE = 15
ACTION_FILESAVEAS = 20
ACTION_FILECLOSE = 25
ACTION_EDITUNDO = 260
ACTION_EDITREDO = 270
ACTION_EDITCUT = 310
ACTION_EDITCOPY = 320
ACTION_EDITPASTE = 330
ACTION_EDITDELETE = 340
ACTION_EDITSELECTALL = 350
ACTION_VIEW = 500
ACTION_EDIT = 505
ACTION_OK = 550
ACTION_CANCEL = 660
ACTION_REFRESH = 880
ACTION_ADD = 1024
ACTION_REMOVE = 2048
ACTION_APPLY = 8192
ACTION_COMMIT = 8288
ACTION_ROLLBACK = 8298
ACTION_PREFS = 41000
ACTION_ABOUT = 49152
ACTION_ABOUTTOOLKIT = 49160
ACTION_EXIT = 65536

# Dummy object to signal new "auto-target-finding" API in specs

AUTO_TARGET = object()

# color constants -- we choose values to make hacks easier :)
COLORNAMES = [
    'BLACK',
    'DARKRED', 'RED',  # 'LIGHTRED', # FIXME: for some reason this isn't in the standard X11 list
    'DARKGREEN', 'GREEN', 'LIGHTGREEN',
    'DARKBLUE', 'BLUE', 'LIGHTBLUE',
    'YELLOW', 'MAGENTA', 'CYAN',
    'DARKGRAY', 'GRAY', 'LIGHTGRAY', 'WHITE'
]
for color in COLORNAMES:
    setattr(sys.modules[__name__], 'COLOR_{}'.format(color), color)

# top window geometry constants
SIZE_NONE = 0
SIZE_CLIENTWRAP = 1
SIZE_MAXIMIZED = 2
SIZE_OFFSET = 4
SIZE_SETTINGS = 8

MOVE_NONE = 0
MOVE_CENTER = 1
MOVE_SETTINGS = 2

# Socket notifier constants
NOTIFY_READ = 0
NOTIFY_WRITE = 1


# Convenience function to add constants

def add_constants(**kw):
    for name, value in kw.items():
        setattr(sys.modules[__name__], name, value)



#!/usr/bin/env python3
"""
PXMLVIEW.PY
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

A simple read-only XML file viewer using the PLIB3 package.
"""

import sys
import os
import xml.etree.ElementTree as etree

from plib.stdlib.coll import basecontainer

from plib.gui import __version__
from plib.gui import main as gui
from plib.gui.defs import *


block_elements = ["p"]  # NOT a complete list, just enough to work with test XML files


def tagtype(node):
    if node.tag in block_elements:
        return "markup block"
    return "element"


def is_leaf(node):
    return len(node) < 1


def cdata(node):
    return node.text


class ETreeNodeWrapper(basecontainer):
    # Ensures that the node objects seen by the list view items will work
    # like standard sequences (etree nodes have some peculiarities since
    # they are a sort of hybrid of sequence and mapping); note that this
    # is a read-only view since we are not doing any XML editing here
    
    def __init__(self, node):
        self._node = node
        basecontainer.__init__(self)
    
    def _indexlen(self):
        return len(self._node)
    
    def _get_data(self, index):
        # Wrap child nodes the same way
        return ETreeNodeWrapper(self._node[index])


class XMLListViewItem(gui.PListViewItem):
    
    def __init__(self, parent, index, node):
        if isinstance(node, ETreeNodeWrapper):
            # Parse node info into listview columns
            cols = [
                " ".join([node._node.tag] + [
                    "{}='{}'".format(key, node._node.get(key))
                    for key in list(node._node.keys())
                ]),
                tagtype(node._node)
            ]
            if is_leaf(node._node):
                s = cdata(node._node)
                childlist = [s] if s else None
            else:
                childlist = node
            data = (cols, childlist)
        else:
            # Make character data look like a child node
            data = ([node, "cdata"], None)
        gui.PListViewItem.__init__(self, parent, index, data)
        self.expand()


class XMLListView(gui.PListView):
    
    itemclass = XMLListViewItem
    
    def __init__(self, parent, data):
        self._filename = data
        self._xml = etree.parse(data)
        gui.PListView.__init__(
            self, parent,
            [
                gui.PHeaderLabel("XML"),
                gui.PHeaderLabel("Node Type", 150, ALIGN_CENTER)
            ],
            [ETreeNodeWrapper(self._xml.getroot())],
            self.current_changed
        )
        if sys.platform == 'darwin':
            fontsize = 16
        else:
            fontsize = 12
        self.set_font("Arial", fontsize)
        self.set_header_font("Arial", fontsize, bold=True)
    
    def current_changed(self, item):
        self._parent.print_status(item.cols[0])


class XMLTabWidget(gui.PTabWidget):
    
    aboutdata = {
        'name': "PXMLViewer",
        'version': __version__,
        'copyright': "Copyright (C) 2008-2015 by Peter A. Donis",
        'license': "GNU General Public License (GPL) Version 2",
        'description': "XML File Viewer",
        'developers': ["Peter Donis"],
        'website': "http://www.peterdonis.net",
        'icon': os.path.join(os.path.split(os.path.realpath(__file__))[0],
                             "pxmlview.png")
    }
    
    actionflags = [
        ACTION_FILEOPEN, ACTION_ABOUT, ACTION_ABOUTTOOLKIT,
        ACTION_EXIT
    ]
    
    defaultcaption = "XML File Viewer"
    large_icons = True
    placement = (SIZE_MAXIMIZED, MOVE_NONE)
    queryonexit = False
    
    def __init__(self, parent):
        # Some gymnastics to help the file open dialog find our test XML files
        realfile = __file__
        if os.path.islink(realfile):
            realfile = os.path.realpath(realfile)
        self._path = os.path.dirname(realfile)
        
        self.mainwin = parent
        if len(sys.argv) > 1:
            filenames = sys.argv[1:]
        else:
            filenames = []
        if not filenames:
            filename = self.get_filename()
            if not filename:
                sys.exit("You must select an XML file to view.")
            filenames.append(filename)
        gui.PTabWidget.__init__(self, parent, None, self.tab_changed)
        for filename in filenames:
            self.add_file(filename)
        parent.connectaction(ACTION_FILEOPEN, self.open_file)
    
    def get_filename(self):
        fname = self.mainwin.getfiledialog(path=self._path)
        if len(fname) > 0:
            fname = str(fname)
            self._path = os.path.dirname(fname)
            return fname
        return ""
    
    def add_file(self, fname):
        widget = XMLListView(self, fname)
        self.append((os.path.basename(fname), widget))
        if len(self) > 1:
            self.set_current_index(len(self) - 1)
    
    def open_file(self):
        fname = self.get_filename()
        if len(fname) > 0:
            self.add_file(fname)
    
    def tab_changed(self, index):
        self.print_status(self.current_widget()._filename)
    
    def print_status(self, text):
        self._parent.statusbar.set_text(text)


if __name__ == "__main__":
    gui.runapp(XMLTabWidget, use_mainwindow=True)



#!/usr/bin/env python3
"""
Sub-Package GUI of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This sub-package contains a complete GUI application
framework which can wrap any of several GUI toolkits.
The main purpose of the framework is to:

(1) Execute common definitions for GUI objects by choosing
    the appropriate ones for the chosen GUI toolkit;
(2) Provide a few final pieces of common functionality for
    GUI applications.

The overall goal is that GUI applications can simply import the
plib.gui.main module and use its classes without requiring any
knowledge of the GUI toolkit being used, other than allowing the
user to choose it. Choosing a toolkit can be done by setting the
environment variable 'GUI_TOOLKIT' to 1, 2, 3, 4, or 5, to correspond
to the constants defined in plib.gui.defs; alternatively, the
variable can be set to one of the string names 'GUI_QT', 'GUI_KDE',
'GUI_GTK', 'GUI_WX', or 'GUI_QT4', which will then be mapped to their
corresponding values. If no such environment variable is found, or
if its value does not correspond to either of the above possibilities,
various other environment variables and other possibilities are
checked to try to determine whether the script is running under KDE
or Gnome; if KDE, the GUI toolkit defaults to KDE if PyKDE is present,
otherwise Qt; if Gnome, it defaults to GTK. If none of the above is
successful, the toolkit defaults to wxWidgets.

Note that the _setup module is used to determine which toolkits are
installed on the system; since this changes very rarely, one of the
PLIB3 post-install scripts generates that module so the checking
doesn't have to be done at runtime every time the GUI sub-package
is used.
"""

__version__ = "0.9.7"



#!/usr/bin/env python3
"""
Module APP -- GUI Application Classes
Sub-Package GUI.BASE of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

Defines the classes that form the basic GUI application framework.
"""

import sys
import os.path

from plib.stdlib.builtins import inverted
from plib.stdlib.classtools import Singleton
from plib.stdlib.decotools import cached_property
from plib.stdlib.ini.defs import *

from plib.gui.defs import *


class PBaseWindowBase(object):
    """Base class for "window" widgets.
    
    This class is mainly for inheritance purposes; it implements
    the basic window behaviors, but does not have all the extra
    stuff in ``PTopWindow`` that relates specifically to being an
    application's main window.
    """
    
    clientwidgetclass = None
    
    defaultcaption = "Window"
    placement = (SIZE_NONE, MOVE_NONE)
    sizeoffset = 160
    
    _clientattrs = ('defaultcaption', 'placement', 'sizeoffset')
    
    def __init__(self, parent, cls):
        self.shown = False
        
        # Figure out if parent is another window or the application
        # (normally only a PTopWindow will have the application as
        # its parent directly, but we implement the check here
        # because it's easier)
        if hasattr(parent, 'app'):
            self.app = parent.app
            self._parent = parent
        elif isinstance(parent, PApplicationBase):
            self.app = parent
            self._parent = None
        else:
            # This shouldn't happen, but just in case...
            self.app = None
            self._parent = None
        
        self.set_caption(self.defaultcaption)
        
        self._set_client_class(cls)
        
        self.clientwidget = self.createclient()
    
    def set_caption(self, caption):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def _set_client_class(self, cls):
        if cls is not None:
            self.clientwidgetclass = cls
        if self.clientwidgetclass is not None:
            cls = self.clientwidgetclass
            for attrname in self._clientattrs:
                if hasattr(cls, attrname):
                    setattr(self, attrname, getattr(cls, attrname))
                elif not hasattr(self, attrname):
                    setattr(self, attrname, None)
    
    def createclient(self):
        """Create the client widget if its class is given.
        """
        
        if self.clientwidgetclass is not None:
            return self.clientwidgetclass(self)
        return None
    
    def get_clientsize(self):
        """Return tuple of (width, height) needed to wrap client.
        """
        c = self.clientwidget
        return (c.preferred_width(), c.preferred_height())
    
    def sizetoscreen(self, maximized=False):
        """Size the window to the screen.
        
        Size the window to be sizeoffset pixels from each screen edge,
        or maximized if the parameter is True.
        """
        raise NotImplementedError
    
    def sizetoclient(self, clientwidth, clientheight):
        """Size the window to a client widget.
        
        Size the window to fit the given client width and height.
        """
        raise NotImplementedError
    
    def center(self):
        """Center the window in the primary screen.
        """
        raise NotImplementedError
    
    def init_placement(self):
        size, pos = self.placement
        if size == SIZE_CLIENTWRAP:
            self.sizetoclient(*self.get_clientsize())
        elif size in (SIZE_MAXIMIZED, SIZE_OFFSET):
            self.sizetoscreen(size == SIZE_MAXIMIZED)
        if (pos == MOVE_CENTER) and (size != SIZE_MAXIMIZED):
            self.center()
    
    def show_init(self):
        """Should always call from derived classes to ensure proper setup.
        """
        if not self.shown:
            # Do placement just before showing for first time
            self.init_placement()
            self.shown = True
    
    def clientcanclose(self):
        return (not hasattr(self.clientwidget, 'acceptclose')) \
            or self.clientwidget.acceptclose()
    
    def acceptclose(self):
        """Return False if window should not close based on current state.
        """
        return self.clientcanclose()
    
    def _canclose(self):
        # Internal method, should not be overridden (override acceptclose
        # to add/modify external behavior)
        return self.acceptclose()
    
    def exit(self):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError


class _PDialogBase(Singleton):
    
    def _init(self, parent):
        self._parent = parent


class PMessageBoxMeta(type):
    """Metaclass to automatically set up message box classes.
    """
    
    def __init__(cls, name, bases, dict):
        # Add None to question map and set up answer map
        type.__init__(cls, name, bases, dict)
        questionmap = getattr(cls, 'questionmap')
        questionmap.update({answerNone: None})
        setattr(cls, 'questionmap', questionmap)
        answermap = inverted(questionmap)
        setattr(cls, 'answermap', answermap)


class PMessageBoxBase(_PDialogBase, metaclass=PMessageBoxMeta):
    
    questionmap = {}
    
    def _messagebox(self, type, caption, text,
                    button1, button2=None, button3=None):
        """Placeholder for derived classes to implement.
        """
        return None
    
    def _translate(self, type, caption, text,
                   button1, button2=answerNone, button3=answerNone):
        
        return self.answermap[self._messagebox(
            type, caption, text,
            self.questionmap[button1],
            self.questionmap[button2],
            self.questionmap[button3])]
    
    def info(self, caption, text):
        """Information message box.
        """
        return self._translate(MBOX_INFO, caption, text,
                               answerOK)
    
    def warn(self, caption, text):
        """Warning message box.
        """
        return self._translate(MBOX_WARN, caption, text,
                               answerOK)
    
    def error(self, caption, text):
        """Error message box.
        """
        return self._translate(MBOX_ERROR, caption, text,
                               answerOK)
    
    def query2(self, caption, text):
        """OK/Cancel message box.
        """
        return self._translate(MBOX_QUERY, caption, text,
                               answerOK, answerCancel)
    
    def query3(self, caption, text):
        """Yes/No/Cancel message box.
        """
        return self._translate(MBOX_QUERY, caption, text,
                               answerYes, answerNo, answerCancel)


class PFileDialogBase(_PDialogBase):
    
    def openfiledialog(self, path, filter):
        """Placeholder for derived classes to implement.
        """
        return ""
    
    def savefiledialog(self, path, filter):
        """Placeholder for derived classes to implement.
        """
        return ""


class PAboutDialogBase(object):
    """Base class for about dialogs.
    
    Implements mapping of standard fields in about data to methods
    of about dialog that will process the data, by hacking
    ``__getattribute__`` to substitute attribute names on the fly.
    """
    
    attrmap = {}
    
    def __init__(self, parent):
        self.mainwidget = parent
    
    def __getattribute__(self, name):
        # Monkeypatch attribute name if necessary
        attrmap = super(PAboutDialogBase, self).__getattribute__('attrmap')
        if name in attrmap:
            name = attrmap[name]
        return super(PAboutDialogBase, self).__getattribute__(name)


def _munge_prefsdata(prefsdata):
    # For re-ordering prefs data if the more readable
    # ordering was used (see PTopWindowBase constructor below)
    return (prefsdata[0], prefsdata[2], prefsdata[1])


def _get_settings(prefsdata):
    # Only call after ordering has been munged if needed (as above)
    from plib.stdlib.ini import PIniFile
    prefsname, specs, _ = prefsdata
    inispecs = [
        (sname, [(oname, otype, default) for oname, otype, default, __ in opts])
        for sname, _, opts in specs
    ]
    return PIniFile(prefsname, inispecs)


def _fix_prefsdata(settings, prefsdata):
    # Only call after _get_settings (as above)
    _, specs, style = prefsdata
    dlgspecs = []
    for sname, title, opts in specs:
        dlgspecs.append((sname, title))
        dlgspecs.extend(
            ("{}_{}".format(sname, oname), label)
            for oname, _, __, label in opts
        )
    return settings, dict(dlgspecs), style


window_optnames = ("left", "top", "width", "height")


class PTopWindowBase(PBaseWindowBase):
    """Base class for 'top window' widgets.
    
    A top window is a 'plain' main application window; it has no
    frills like menus, toolbars, status bars, etc. built in (if
    you want those frills, use PMainWindow instead). It does have
    some basic functionality, however, using the following class
    fields:
    
    clientwidgetclass -- gives the class of the client widget (actually can
    be any callable with the right signature that returns a widget); if None,
    no client widget is created automatically (widgets can still be created
    manually in user code). The callable must take one argument, which will
    be the PTopWindow instance creating it.
    
    Note that all the rest of the options below can be read from a client
    widget class, so the need to set them by subclassing PTopWindow directly
    should be rare:
    
    aboutdialogclass -- gives class to be used to display the 'about' dialog
    box. This class is normally set internally to PLIB3 and should not need
    to be overridden by the user.
    
    messageboxclass -- gives the class to be used for message dialogs; normally
    set internally by PLIB3.
    
    filedialogclass -- gives the class to be used for file open/save dialogs;
    normally set internally by PLIB3.
    
    aboutdata -- gives data for display in the 'about' dialog.
    
    prefsdata -- gives parameters for constructing the 'preferences' dialog.
    
    defaultcaption -- gives the caption if no editor object is found
    
    placement -- how the window should be sized (normal, maximized, wrapped to
    the client widget, or offset from the screen edge) and positioned on the
    screen (centered, or left to the system's default positioning)
    
    sizeoffset -- how many pixels from the edge of the screen this window
    should be sized (only used if placement indicates sizing to offset from
    screen edge)
    """
    
    messageboxclass = None
    filedialogclass = None
    aboutdialogclass = None
    
    aboutdata = {}
    prefsdata = None
    
    abouttoolkitfunc = None
    
    defaultcaption = "Top Window"
    
    _clientattrs = PBaseWindowBase._clientattrs + (
        'messageboxclass', 'filedialogclass', 'aboutdata', 'prefsdata'
    )
    
    def __init__(self, parent, cls=None):
        PBaseWindowBase.__init__(self, parent, cls)
        self._setup_aboutdata()
        self._setup_prefsdata()
    
    def get_basename(self):
        """Return the base name of the application script.
        """
        return os.path.splitext(os.path.basename(sys.argv[0]))[0]
    
    def _setup_aboutdata(self):
        if 'icon' in self.aboutdata:
            self.set_iconfile(self.aboutdata['icon'])
    
    def _setup_prefsdata(self):
        size, pos = self.placement
        if self.prefsdata:
            # Support alternate ordering of prefs parameters
            if isinstance(self.prefsdata[1], int):
                # Re-order to put the section type flag after the
                # list of labels (having the list last is more
                # readable in the class definition, but the actual
                # dialog constructor wants the flag last)
                self.prefsdata = _munge_prefsdata(self.prefsdata)
            prefsdata = self.prefsdata
        elif (size == SIZE_SETTINGS) or (pos == MOVE_SETTINGS):
            # Create an empty prefsdata object so we can add the
            # settings for window coordinates below
            prefsdata = (
                self.get_basename(), [], SECTION_GROUPBOX)
        else:
            prefsdata = None
        if prefsdata:
            # Add the window placement settings if the class fields
            # for placement are set appropriately
            if (size == SIZE_SETTINGS) or (pos == MOVE_SETTINGS):
                prefsdata[1].append(
                    ("window", "Window Placement", [
                        (optname, INI_INTEGER, -1, optname.capitalize())
                        for optname in window_optnames
                    ])
                )
            # Now split out settings from prefsdata if a settings
            # name is given instead of a PIniFile instance
            if isinstance(prefsdata[0], str):
                self.settings = _get_settings(prefsdata)
                if self.prefsdata:
                    self.prefsdata = _fix_prefsdata(self.settings, prefsdata)
            else:
                self.settings = prefsdata[0]
        else:
            self.settings = None
    
    def set_iconfile(self, iconfile):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    @cached_property
    def messagebox(self):
        if self.messageboxclass is not None:
            return self.messageboxclass(self)
        return None
    
    @cached_property
    def filedialog(self):
        if self.filedialogclass is not None:
            return self.filedialogclass(self)
        return None
    
    @cached_property
    def prefsdialog(self):
        if self.prefsdata:
            # This avoids circular imports when this module is first loaded
            from plib.gui import main as gui
            return gui.PPrefsDialog(self, *self.prefsdata)
        return None
    
    def _size_to_settings(self, width, height):
        """Size the window to the given width and height (including frame).
        """
        raise NotImplementedError
    
    def _move_to_settings(self, left, top):
        """Move the window to the given position (including frame).
        """
        raise NotImplementedError
    
    def _get_current_geometry(self):
        """Return the window's current geometry (including frame).
        """
        raise NotImplementedError
    
    def set_settings_placement(self, setsize, setpos):
        # Set previous window placement from settings
        if setsize or setpos:
            left, top, width, height = tuple(
                getattr(self.settings, "window_{}".format(optname))
                for optname in window_optnames
            )
            if setsize:
                self._size_to_settings(width, height)
            if setpos:
                self._move_to_settings(left, top)
    
    def get_settings_placement(self, setsize, setpos):
        # Store current window size and position in settings
        if setsize or setpos:
            left, top, width, height = self._get_current_geometry()
            for optname in window_optnames:
                setattr(
                    self.settings,
                    "window_{}".format(optname),
                    locals()[optname]
                )
    
    def get_coordinates(self):
        """Update settings with current coordinates.
        
        Called by PPrefsDialog before showing the dialog.
        """
        size, pos = self.placement
        self.get_settings_placement(
            size == SIZE_SETTINGS, pos == MOVE_SETTINGS)
    
    def set_coordinates(self):
        """Size/move window to coordinates in settings.
        
        Called by PPrefsDialog when Apply or OK is pressed.
        """
        size, pos = self.placement
        self.set_settings_placement(
            size == SIZE_SETTINGS, pos == MOVE_SETTINGS)
    
    def init_placement(self):
        PBaseWindowBase.init_placement(self)
        self.set_coordinates()
    
    def choose_directory(self, curdir):
        """Display select folder dialog and return chosen folder.
        """
        raise NotImplementedError
    
    def getfiledialog(self, path="", filter="", action=ACTION_FILEOPEN):
        if action == ACTION_FILEOPEN:
            return self.filedialog.openfiledialog(path, filter)
        if action == ACTION_FILESAVEAS:
            return self.filedialog.savefiledialog(path, filter)
        return ""
    
    def preferences(self):
        if self.prefsdialog:
            self.prefsdialog.showdialog()
    
    def about(self):
        if (self.aboutdata is not None) and \
                (self.aboutdialogclass is not None):
            
            dialog = self.aboutdialogclass(self)
            for key, item in self.aboutdata.items():
                getattr(dialog, key)(item)
            dialog.display()
    
    def about_toolkit(self):
        if self.abouttoolkitfunc is not None:
            self.abouttoolkitfunc()
    
    def _canclose(self):
        result = PBaseWindowBase._canclose(self)
        if result and self.settings:
            self.get_coordinates()
            self.settings.writeini()
        return result


class PApplicationBase(object):
    """Base class for GUI application.
    
    Automatically sizes the main widget and centers it in
    the primary screen if the widget's class flags are set
    appropriately (see PTopWindow).
    
    Descendant app classes should set the class variable
    ``mainwidgetclass`` to the appropriate class object. If this
    is the only customization you want to do, however, you do
    not need to subclass this class--just pass your main window
    class derived from ``PTopWindow`` to the ``runapp`` function;
    see that function's docstring. In fact, you can even pass a
    client widget class to ``runapp``, and it will automatically
    be wrapped in a ``PTopWindow``--or, if you set the keyword
    argument ``use_mainwindow`` to ``True``, a ``PMainWindow``.
    See the ``pyidserver-gui`` and ``scrips-edit`` example
    programs for typical usage.
    
    The only time you should need to subclass this class is to
    override ``createMainWidget`` (to alter the parameters passed
    to the main widget, or do other processing after it's created
    but before the rest of ``__init__``) or to provide other
    functionality that has to be at the application level rather
    than in the main widget (but this should be extremely rare).
    """
    
    mainwidgetclass = None
    
    def __init__(self, arglist=[], cls=None, use_mainwindow=False):
        self.arglist = arglist
        self.mainwin = None
        
        # Set up main widget class
        if self.mainwidgetclass is None:
            # Import here to avoid circular imports; note that we
            # should normally get here only if the ``cls`` parameter
            # is not None (so we have a real client class)
            from plib.gui import main as gui
            if use_mainwindow:
                self.mainwidgetclass = gui.PMainWindow
            else:
                self.mainwidgetclass = gui.PTopWindow
        
        # Set up main application class
        if cls is not None:
            # The ``cls`` parameter should be either a valid main widget
            # class (which will create its own client widget), or a valid
            # client widget class (which will be wrapped in the default
            # main widget)
            self.mainwinclass = cls
        else:
            self.mainwinclass = self.mainwidgetclass
    
    def createMainWidget(self):
        """Create the main widget and return it.
        """
        if issubclass(self.mainwinclass, PTopWindowBase):
            return self.mainwinclass(self)
        return self.mainwidgetclass(self, self.mainwinclass)
    
    def _eventloop(self):
        """Placeholder for derived classes for main event loop.
        """
        raise NotImplementedError
    
    def run(self):
        """Show the main widget and run the main event loop.
        """
        
        self.mainwin.show_init()
        self._eventloop()
    
    def process_events(self):
        """Placeholder for derived classes to pump events outside main loop.
        """
        raise NotImplementedError
    
    def before_quit(self):
        """Placeholder for derived classes if needed.
        
        Note that this method cannot assume that *any* objects other than
        the application itself are still available; things that need to be
        done with widgets still available should be done in response to
        the main widget's SIGNAL_CLOSING signal.
        """
        pass



#!/usr/bin/env python3
"""
Module QT4COMMON -- Python Qt 4 Common Objects
Sub-Package GUI.TOOLKITS.QT4 of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains common Qt 4 GUI objects for use by the other
Qt 4 modules.
"""

from abc import ABCMeta

from PyQt4 import Qt as qt

from plib.gui.defs import *

_qtalignmap = {
    ALIGN_LEFT: qt.Qt.AlignLeft | qt.Qt.AlignVCenter,
    ALIGN_CENTER: qt.Qt.AlignCenter,
    ALIGN_RIGHT: qt.Qt.AlignRight | qt.Qt.AlignVCenter
}

_qtcolormap = dict(
    (color, qt.QColor(color.lower()))
    for color in COLORNAMES
)

_qtmessagefuncs = {
    MBOX_INFO: qt.QMessageBox.information,
    MBOX_WARN: qt.QMessageBox.warning,
    MBOX_ERROR: qt.QMessageBox.critical,
    MBOX_QUERY: qt.QMessageBox.question
}

_qtsignalmap = {
    SIGNAL_ACTIVATED: "triggered()",
    SIGNAL_CLICKED: "clicked()",
    SIGNAL_TOGGLED: "toggled(bool)",
    SIGNAL_SELECTED: "activated(int)",
    SIGNAL_LISTSELECTED: "currentItemChanged(QTreeWidgetItem*, QTreeWidgetItem*)",
    SIGNAL_CELLSELECTED: "currentCellChanged(int, int, int, int)",
    SIGNAL_TABLECHANGED: "cellChanged(int, int)",
    SIGNAL_TEXTCHANGED: "textChanged()",  # textEdited()?
    SIGNAL_EDITCHANGED: "textChanged(const QString&)",  # textEdited(const QString&)?
    SIGNAL_ENTER: "returnPressed()",
    SIGNAL_TABCHANGED: "currentChanged(int)",
    SIGNAL_NOTIFIER: "activated(int)",
    SIGNAL_BEFOREQUIT: "aboutToQuit()"
}

_qteventmap = {
    SIGNAL_CELLCHANGED: "tableCellChanged",
    SIGNAL_TEXTSTATECHANGED: "textStateChanged",
    SIGNAL_FOCUS_IN: "focusInEvent",
    SIGNAL_FOCUS_OUT: "focusOutEvent",
    SIGNAL_CLOSING: "closeEvent",
    SIGNAL_SHOWN: "showEvent",
    SIGNAL_HIDDEN: "hideEvent"
}


def _qtmap(signal):
    if signal in _qtsignalmap:
        return qt.SIGNAL(_qtsignalmap[signal])
    elif signal in _qteventmap:
        return qt.SIGNAL(_qteventmap[signal])
    else:
        return None


# Ugly hack to fix metaclass conflict for classes that use the collection
# ABCs

_QtMeta = type(qt.QObject)


class _PQtMeta(ABCMeta, _QtMeta):
    
    def __init__(cls, name, bases, attrs):
        _QtMeta.__init__(cls, name, bases, attrs)
        ABCMeta.__init__(cls, name, bases, attrs)


# NOTE: we don't need to define 'wrapper' methods here as we do under GTK and
# wxWidgets because Qt silently discards any extra parameters that are not
# accepted by a signal handler. (BTW, this is good because wrappers don't seem
# to work like they should in Qt -- see PEDIT.PY, PEditor._setup_signals.)

class _PQtCommunicator(object):
    """Mixin class to abstract signal/slot functionality in Qt.
    """
    
    def setup_notify(self, signal, target):
        qt.QObject.connect(self, _qtmap(signal), target)
    
    def do_notify(self, signal, *args):
        sig = _qtmap(signal)
        if sig is not None:
            self.emit(sig, *args)
    
    _emit_event = do_notify


class _PQtWidgetBase(object):
    """Mixin class to provide common Qt widget methods.
    """
    
    fn_enable_get = 'isEnabled'
    fn_enable_set = 'setEnabled'
    
    _palette = None
    
    def update_widget(self):
        self.update()
    
    def preferred_width(self):
        return max(self.minimumSize().width(), self.sizeHint().width())
    
    def preferred_height(self):
        return max(self.minimumSize().height(), self.sizeHint().height())
    
    def set_width(self, width):
        self.resize(width, self.height())
    
    def set_height(self, height):
        self.resize(self.width(), height)
    
    def set_position(self, left, top):
        if not (None in (left, top)):
            self.move(left, top)
    
    def _mapped_color(self, color):
        if isinstance(color, qt.QColor):
            return color
        return _qtcolormap[color]
    
    def set_colors(self, fg=None, bg=None):
        palette = qt.QPalette(self.palette())
        if fg is not None:
            palette.setColor(self.foregroundRole(), self._mapped_color(fg))
        if bg is not None:
            self.setAutoFillBackground(True)
            palette.setColor(self.backgroundRole(), self._mapped_color(bg))
        self.setPalette(palette)
    
    def set_foreground_color(self, color):
        self.set_colors(fg=color)
    
    def set_background_color(self, color):
        self.set_colors(bg=color)
    
    def get_font_name(self):
        return self.font().family()
    
    def get_font_size(self):
        return self.font().pointSize()
    
    def get_font_bold(self):
        return self.font().bold()
    
    def get_font_italic(self):
        return self.font().italic()
    
    def _qt_font_object(self, font_name, font_size, bold, italic):
        font = qt.QFont(font_name, font_size)
        font.setBold(bold)
        font.setItalic(italic)
        return font
    
    def _set_font_object(self, font_name, font_size, bold, italic):
        self.setFont(self._qt_font_object(
            font_name, font_size, bold, italic))
    
    def set_focus(self):
        self.setFocus()


class _PQtWidget(_PQtCommunicator, _PQtWidgetBase):
    """Mixin class for Qt widgets that can send/receive signals.
    """
    
    def focusInEvent(self, event):
        self.widget_class.focusInEvent(self, event)
        self._emit_event(SIGNAL_FOCUS_IN)
    
    def focusOutEvent(self, event):
        self.widget_class.focusOutEvent(self, event)
        self._emit_event(SIGNAL_FOCUS_OUT)


class _PQtClientWidget(_PQtWidget):
    """Mixin class for Qt main window client widgets.
    """
    pass



#!/usr/bin/env python3
"""
Module WXDISPLAY -- Python wxWidgets Text Display Widgets
Sub-Package GUI.TOOLKITS.WX of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the wxWidgets GUI objects for text display widgets.
"""

import wx

from plib.gui.defs import *
from plib.gui._widgets import display

from ._wxcommon import _PWxWidget, _wx_ec_scroll_style


class PWxTextDisplay(_PWxWidget, wx.TextCtrl, display.PTextDisplayBase):
    
    _style = wx.TE_MULTILINE | wx.TE_PROCESS_TAB
    
    def __init__(self, parent, text=None, geometry=None, scrolling=False):
        self._expand = True
        self._align = ALIGN_JUST
        if scrolling:
            self._style = self._style | _wx_ec_scroll_style
        wx.TextCtrl.__init__(self, parent, style=self._style)
        self.SetEditable(False)
        display.PTextDisplayBase.__init__(self, text, geometry)
    
    def get_text(self):
        return self.GetValue()
    
    def set_text(self, value):
        self.SetValue(value)



#!/usr/bin/env python3
"""
Module NOTIFY -- Base for GUI Signal Emitters
Sub-Package GUI.BASE of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

Defines an abstract base class for all signal-emitting GUI classes.
"""

from plib.gui.common import *


class _PNotifyBase(object):
    """Base class for objects that can send signals to notify other objects.
    """
    
    signal = None
    
    def __init__(self, target=None):
        if target is not None:
            self.connect_to(target)
    
    def connect_to(self, target):
        """Connect our standard signal to the target.
         
        Note that some other derived or mixin class must implement
        the setup_notify method.
        """
        
        if self.signal:
            self.setup_notify(self.signal, target)
    
    def setup_notify(self, signal, target):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def do_notify(self, signal, *args):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def get_icon_filename(key):
        return pxfile(actiondict[key][0])
    get_icon_filename = staticmethod(get_icon_filename)
    
    def get_menu_str(key):
        return actiondict[key][1]
    get_menu_str = staticmethod(get_menu_str)
    
    def get_toolbar_str(key):
        return actiondict[key][1].replace('&', '').strip('.')
    get_toolbar_str = staticmethod(get_toolbar_str)
    
    def get_accel_str(key):
        s = actiondict[key][1]
        i = s.find('&')
        if i > -1:
            return "+".join(["Alt", s[i + 1].upper()])
        else:
            return None
    get_accel_str = staticmethod(get_accel_str)



#!/usr/bin/env python3
"""
Module WXTABWIDGET -- Python wxWidgets Tab Widget
Sub-Package GUI.TOOLKITS.QT of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the wxWidgets GUI objects for the tab widget.
"""

import wx

from plib.gui.defs import *
from plib.gui._widgets import tabwidget

from ._wxcommon import _PWxClientWidget


class PWxTabWidget(_PWxClientWidget, wx.Notebook, tabwidget.PTabWidgetBase):
    
    def __init__(self, parent, tabs=None, target=None):
        wx.Notebook.__init__(self, parent)
        self._align = ALIGN_JUST  # used by PWxPanel to determine placement
        self._expand = True
        tabwidget.PTabWidgetBase.__init__(self, parent, tabs, target)
    
    def __len__(self):
        return self.GetPageCount()
    
    def _get_tabtitle(self, index):
        return self.GetPageText(index)
    
    def _set_tabtitle(self, index, title):
        self.SetPageText(index, title)
    
    def _add_tab(self, index, title, widget):
        if index == self.__len__():
            self.AddPage(widget, title)
        else:
            self.InsertPage(index, widget, title)
    
    def _del_tab(self, index):
        self.RemovePage(index)
    
    def current_index(self):
        return self.GetSelection()
    
    def set_current_index(self, index):
        self.SetSelection(index)



#!/usr/bin/env python3
"""
Module notifier_io -- PLIB3.IO mixins for Notifier Client
Sub-Package GUI of Package PLIB3
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information
"""

from plib.gui import main as gui

if hasattr(gui, 'PSocketNotifier'):  # Qt 3/4 and KDE 3/4
    
    class NotifierClientMixin(object):
        
        done_callback = None
        yielded = False
        
        def do_connect(self, addr):
            super(NotifierClientMixin, self).do_connect(addr)
            if self.connected or self.connect_pending:
                self.init_notifiers()
        
        def start(self, data):
            super(NotifierClientMixin, self).start(data)
            self.check_notifiers()
        
        def _doyield(self):
            self.yielded = True
            super(NotifierClientMixin, self)._doyield()
        
        def do_loop(self, callback=None):
            """Override to yield back to the GUI event loop.
            """
            self.done_callback = callback
            if not self.yielded:
                self._doyield()
        
        def _unyield(self):
            self.done_callback = None
            self.yielded = False
            super(NotifierClientMixin, self)._unyield()
        
        def handle_write(self):
            super(NotifierClientMixin, self).handle_write()
            self.check_notifiers()
        
        def handle_read(self):
            super(NotifierClientMixin, self).handle_read()
            self.check_notifiers()
        
        def check_done(self):
            """Override to un-yield from the GUI event loop if done.
            """
            super(NotifierClientMixin, self).check_done()
            if self.yielded and (((self.done_callback is not None) and
                                  (self.done_callback() is False)) or
                                 self.done):
                self._unyield()
        
        def close(self):
            """Override to ensure we un-yield when closed.
            """
            super(NotifierClientMixin, self).close()
            if self.yielded:
                self._unyield()
        
        def handle_close(self):
            self.del_notifiers()
            super(NotifierClientMixin, self).handle_close()


else:  # GTK and wxWidgets
    
    class NotifierClientMixin(object):
        
        def do_connect(self, addr):
            super(NotifierClientMixin, self).do_connect(addr)
            if self.connected or self.connect_pending:
                self.set_app()
        
        def handle_close(self):
            self.clear_app()
            super(NotifierClientMixin, self).handle_close()



#!/usr/bin/env python3
"""
Module notifier_asyncore -- ASYNCORE mixins for Notifier Client
Sub-Package GUI of Package PLIB3
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information
"""

import asyncore

from plib.gui import main as gui

if hasattr(gui, 'PSocketNotifier'):  # Qt 3/4 and KDE 3/4
    
    mixin_channels = {}
    asyncore_patched = False
    old_loop = None
    
    class NotifierClientMixin(object):
        
        def _patch_asyncore(self):
            global asyncore_patched
            global old_loop
            
            if not asyncore_patched:
                old_loop = asyncore.loop
                
                def loop(*args, **kwargs):
                    # Note that we ignore the normal asyncore.loop args; in
                    # particular there is no way to support a timeout on the
                    # loop--this is not considered a major issue since the
                    # whole point is to keep the GUI snappy while handling
                    # socket events; non-GUI and non-socket operations should
                    # be done in separate threads or processes
                    self._doyield()
                
                asyncore.loop = loop
                asyncore_patched = True
            
            mixin_channels[self._fileno] = self
        
        def handle_connect_event(self):
            self.dispatcher_class.handle_connect_event(self)
            if self.connected:
                self._patch_asyncore()
                self.init_notifiers()
        
        def handle_read_event(self):
            self.dispatcher_class.handle_read_event(self)
            if self.connected:
                self.check_notifiers()
        
        def handle_write_event(self):
            self.dispatcher_class.handle_write_event(self)
            if self.connected:
                self.check_notifiers()
        
        def _unpatch_asyncore(self):
            global asyncore_patched
            global old_loop
            
            # Only shut down the GUI event loop if we were the last channel active
            if (not mixin_channels) and asyncore_patched:
                self._unyield()
                asyncore.loop = old_loop
                asyncore_patched = False
                old_loop = None
        
        def close(self):
            self.del_notifiers()
            if self._fileno in mixin_channels:
                # Need to check first since close may be called multiple times
                del mixin_channels[self._fileno]
            self.dispatcher_class.close(self)
            self._unpatch_asyncore()


else:  # GTK and wxWidgets
    
    class NotifierClientMixin(object):
        
        def handle_connect_event(self):
            self.dispatcher_class.handle_connect_event(self)
            if self.connected:
                self.set_app()
        
        def do_loop(self, process_events):
            while asyncore.map:
                asyncore.loop(self.poll_timeout, use_poll=True)
                process_events()
        
        def close(self):
            self.clear_app()
            self.dispatcher_class.close(self)



#!/usr/bin/env python3
"""
Module WXGROUPBOX -- Python wxWidgets Group Box Widgets
Sub-Package GUI.TOOLKITS.WX of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the wxWidgets GUI objects for group box widgets.
"""

import wx

from plib.gui.defs import *
from plib.gui._widgets import groupbox

from ._wxcommon import _PWxWidget


class PWxGroupBox(_PWxWidget, wx.Panel, groupbox.PGroupBoxBase):
    
    def __init__(self, parent, caption, controls=None,
                 margin=-1, spacing=-1, geometry=None):
        
        wx.Panel.__init__(self, parent, style=wx.NO_BORDER)
        self._align = ALIGN_TOP
        self._expand = True
        self._margin = margin
        self._spacing = spacing
        self._haswidgets = False
        self._box = wx.StaticBox(self)
        self._boxsizer = wx.StaticBoxSizer(self._box, wx.VERTICAL)
        self._sizer = wx.BoxSizer(wx.VERTICAL)
        self._sizer.Add(self._boxsizer, 1, wx.EXPAND, 0)
        groupbox.PGroupBoxBase.__init__(self, parent, caption, controls,
                                        margin, spacing, geometry)
    
    def set_caption(self, caption):
        self._box.SetLabel(caption)
    
    def set_margin(self, margin):
        # Margin is dealt with differently in wxWidgets, see above
        pass
    
    def set_spacing(self, spacing):
        # Spacing is dealt with differently in wxWidgets, see above
        pass
    
    def _add_control(self, control):
        # Somewhat abbreviated version of the cruft in PWxPanel
        
        if self._haswidgets:
            if self._spacing > -1:
                self._boxsizer.AddSpacer(self._spacing)
        elif self._margin > -1:
            self._boxsizer.AddSpacer(self._margin)
        
        if hasattr(control, '_align') and (control._align != ALIGN_JUST):
            proportion = 0
            flag = wx.ALIGN_TOP
        else:
            proportion = 1
            flag = 0
        
        if hasattr(control, '_expand') and control._expand:
            flag |= wx.EXPAND
        
        flag |= wx.ALIGN_CENTER_HORIZONTAL | wx.LEFT | wx.RIGHT
        
        self._boxsizer.Add(control, proportion, flag, self._margin)
        if not self._haswidgets:
            self._haswidgets = True
    
    def _dolayout(self):
        if self._margin > -1:
            self._boxsizer.AddSpacer(self._margin)
        self.SetSizerAndFit(self._sizer)



#!/usr/bin/env python3
"""
Module QT4LABEL -- Python Qt 4 Text Label Widgets
Sub-Package GUI.TOOLKITS.QT4 of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the Qt 4 GUI objects for text label widgets.
"""

from PyQt4 import Qt as qt

from plib.gui._widgets import label

from ._qt4common import _PQtWidget


class PQtTextLabel(_PQtWidget, qt.QLabel, label.PTextLabelBase):
    
    widget_class = qt.QLabel
    
    def __init__(self, parent, text=None, geometry=None):
        qt.QLabel.__init__(self, parent)
        label.PTextLabelBase.__init__(self, text, geometry)
    
    def get_text(self):
        return str(self.text())
    
    def set_text(self, value):
        self.setText(value)
    
    def set_width(self, width):
        self.resize(width, self.height())
        # Qt text labels don't seem to fully respect
        # qt.QSizePolicy.MinimumExpanding
        self.setMinimumWidth(width)



#!/usr/bin/env python3
"""
Module WXMAINWIN -- Python wxWidgets Main Window Objects
Sub-Package GUI.TOOLKITS.WX of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the wxWidgets GUI main window objects.
"""

from plib.gui.defs import *
from plib.gui._base import mainwin

from ._wxapp import _PWxMainMixin
from ._wxaction import PWxMenu, PWxToolBar, PWxAction
from ._wxstatusbar import PWxStatusBar


class PWxMainWindow(_PWxMainMixin, mainwin.PMainWindowBase):
    """Customized wxWidgets main window class.
    """
    
    menuclass = PWxMenu
    toolbarclass = PWxToolBar
    statusbarclass = PWxStatusBar
    actionclass = PWxAction
    
    def __init__(self, parent, cls=None):
        _PWxMainMixin.__init__(self, None)
        mainwin.PMainWindowBase.__init__(self, parent, cls)
        if self.menu is not None:
            self.SetMenuBar(self.menu)
        if self.toolbar is not None:
            self.SetToolBar(self.toolbar)
        if self.statusbar is not None:
            self.SetStatusBar(self.statusbar)
        self._setup_events()
    
    def show_init(self):
        mainwin.PMainWindowBase.show_init(self)
        _PWxMainMixin.show_init(self)



#!/usr/bin/env python3
"""
Module COMBO -- GUI Combo Box Widgets
Sub-Package GUI.WIDGETS of Package PLIB3 -- Python GUI Framework
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

Defines the widget classes for combo box widgets.
"""

from plib.stdlib.coll import baselist

from plib.gui.defs import *
from plib.gui._widgets import _control


class PComboBoxBase(_control._PDialogControl, baselist):
    """
    Combo box that looks like a list of strings.
    
    (Note: currently selection is limited to items added to the
    combo box programmatically; the user cannot edit in the edit
    control and add new items to the pick list.)
    """
    
    signal = SIGNAL_SELECTED
    
    def __init__(self, sequence=None, target=None, geometry=None):
        _control._PDialogControl.__init__(self, target, geometry)
        baselist.__init__(self, sequence)
    
    def current_text(self):
        return self[self.current_index()]
    
    def set_current_text(self, text):
        self.set_current_index(self.index(text))
    
    def current_index(self):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError
    
    def set_current_index(self, index):
        """Placeholder for derived classes to implement.
        """
        raise NotImplementedError



#!/usr/bin/env python3
"""
Module WXACTION -- Python wxWidgets Action Objects
Sub-Package GUI.TOOLKITS.WX of Package PLIB3 -- Python GUI Toolkits
Copyright (C) 2008-2015 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the wxWidgets GUI objects for handling user actions.
"""

import wx

from plib.gui._base import action

from ._wxcommon import _PWxCommunicator, _PWxWidgetBase, _wxstockids


def scaled_bitmap(image, factor):
    if factor is not None:
        image = image.Scale(image.GetWidth() * factor,
                            image.GetHeight() * factor)
    return wx.BitmapFromImage(image)


class PWxPopup(wx.Menu):
    """Customized wxWidgets popup menu class.
    """
    
    def __init__(self, mainwidget):
        self.mainwidget = mainwidget
        astyle = 0
        wx.Menu.__init__(self, "", astyle)


class PWxMenu(_PWxWidgetBase, wx.MenuBar, action.PMenuBase):
    """Customized wxWidgets menu class.
    """
    
    popupclass = PWxPopup
    
    def __init__(self, mainwidget):
        astyle = 0
        wx.MenuBar.__init__(self, astyle)
        action.PMenuBase.__init__(self, mainwidget)
    
    def _add_popup(self, title, popup):
        self.Append(popup, title)
    
    def _add_popup_action(self, act, popup):
        if act._id > wx.ID_HIGHEST:
            item = wx.MenuItem(popup, act._id, act.menustr)
        else:
            item = wx.MenuItem(popup, act._id)
            #act.bitmap = item.GetBitmap()
        item.SetBitmap(scaled_bitmap(act.image, act.menufactor))
        popup.AppendItem(item)


class PWxToolBar(_PWxWidgetBase, wx.ToolBar, action.PToolBarBase):
    """Customized wxWidgets toolbar class.
    """
    
    def __init__(self, mainwidget):
        astyle = 0
        if mainwidget.show_labels:
            astyle = astyle | wx.TB_TEXT
        wx.ToolBar.__init__(self, mainwidget, style=astyle)
        action.PToolBarBase.__init__(self, mainwidget)
        self.Realize()
    
    def add_action(self, act):
        #if act._id > wx.ID_HIGHEST:
        img = scaled_bitmap(act.image, act.toolbarfactor)
        #else:
        #    img = wx.ArtProvider.GetBitmap(act._id, wx.ART_TOOLBAR)
        #    img = act.bitmap
        s = act.toolbarstr
        if self.mainwin.show_labels:
            self.AddLabelTool(act._id, s, img, shortHelp=s)
        else:
            self.AddTool(act._id, img, shortHelpString=s)
    
    def add_separator(self):
        self.AddSeparator()


class PWxAction(_PWxCommunicator, action.PActionBase):
    """Customized wxWidgets action class.
    """
    
    def __init__(self, key, mainwidget):
        self.menufactor = 0.5
        if mainwidget.large_icons:
            self.toolbarfactor = None
        else:
            self.toolbarfactor = 0.6875
        self.menustr = self.get_menu_str(key)
        self.toolbarstr = self.get_toolbar_str(key)
        self.accelstr = self.get_accel_str(key)
        if key in _wxstockids:
            self._id = _wxstockids[key]
        #    self.image = None
        else:
            self._id = wx.ID_HIGHEST + key
        self.image = wx.Image(self.get_icon_filename(key))
        action.PActionBase.__init__(self, key, mainwidget)
    
    # Need these three 'fake' widget methods to enable setup_notify mechanism
    def GetId(self):
        return self._id
    
    def Connect(self, *args):
        self.mainwin.Connect(*args)
    
    def Bind(self, event, target):
        self.mainwin.Bind(event, target, id=self._id)
    
    def enable(self, enabled):
        menu = self.mainwin.menu
        toolbar = self.mainwin.toolbar
        if menu is not None:
            menu.Enable(self._id, enabled)
        if toolbar is not None:
            toolbar.EnableTool(self._id, enabled)
